/**
 *  traffic
 *  Author: Patrick Taillandier
 *  Description: step 5
 */

model traffic

global {
	file shapefile_buildings <- file("../includes/buildings.shp");
	file shapefile_roads <- file("../includes/roads.shp");
	geometry shape <- envelope(shapefile_roads);
	float step <- 10 #s;
	graph road_network;
	
	init {
		create building from: shapefile_buildings with:[height::int(read("height"))];
		create road from: shapefile_roads;
		create inhabitant number: 1000{
			location <- any_location_in(one_of(building));
      	}
      	road_network <- as_edge_graph(road);
	}
	
	
	
	reflex update_speed {
		map<road,float> new_weights <- road as_map (each::each.shape.perimeter * each.speed_rate);
		road_network <- road_network with_weights new_weights;
	}
}

species inhabitant skills: [moving]{
	point target <- nil;
	float proba_leave <- 0.05; 
	float speed <- 5 #km/#h;
	rgb color <- rnd_color(255);
	
	reflex leave when: (target = nil) and (flip(proba_leave)) {
		target <- any_location_in(one_of(building));
	}
	
	reflex move when: target != nil {
		do goto target: target on: road_network;
		if (location = target) {
			target <- nil;
		}	
	}
	
	aspect circle {
		draw circle(5) color: color;
	}
	
	aspect threeD{
		draw pyramid(5) color: color;
		draw sphere(2) at: location + {0,0,5} color: color;
	}
}

species building {
	int height ;
	aspect threeD {
		draw shape color: #gray depth: height 
			texture: ["../includes/roof_top.png","../includes/texture5.jpg"];
	}
}

species road {
	float capacity <- 1 + shape.perimeter/30;
	int nb_drivers <- 0 update: length(inhabitant at_distance 1);
	float speed_rate <- 1.0 update:  exp(-nb_drivers/capacity);
	aspect geom {
		draw (shape + 3 * speed_rate) color: #red;
	}
}

experiment traffic type: gui {
	output {
		display map type: opengl {
			image "../includes/soil.jpg" refresh: false;
			species building aspect: threeD refresh: false;
			species road aspect: geom ;
			species inhabitant aspect: threeD;
		}
	}
}
